@@ -12,6 +12,7 @@ one way to generate any given value... but much more importantly, we can do
 most things by construction instead of by filtering.  That's the difference
 between "I'd like it to be faster" and "doesn't finish at all".
 """
+import contextlib
 import itertools
 import json
 import math
@@ -69,13 +70,15 @@ def next_down(val: float) -> float:
 
 
 def _get_validator_class(schema: Schema) -> JSONSchemaValidator:
-    try:
+    with contextlib.suppress(jsonschema.exceptions.SchemaError):
         validator = jsonschema.validators.validator_for(schema)
         validator.check_schema(schema)
-    except jsonschema.exceptions.SchemaError:
-        validator = jsonschema.Draft4Validator
-        validator.check_schema(schema)
-    return validator
+        return validator
+    with contextlib.suppress(jsonschema.exceptions.SchemaError):
+        jsonschema.Draft7Validator.check_schema(schema)
+        return jsonschema.Draft7Validator
+    jsonschema.Draft4Validator.check_schema(schema)
+    return jsonschema.Draft4Validator
 
 
 def make_validator(schema: Schema) -> JSONSchemaValidator:
@@ -208,7 +211,7 @@ def canonicalish(schema: JSONType) -> Dict[str, Any]:
     This is obviously incomplete, but improves best-effort recognition of
     equivalent schemas and makes conversion logic simpler.
     """
-    if schema is True:
+    if schema is True:  # noqa: SIM114
         return {}
     elif schema is False:
         return {"not": {}}
@@ -242,13 +245,16 @@ def canonicalish(schema: JSONType) -> Dict[str, Any]:
     if_ = schema.pop("if", None)
     then = schema.pop("then", schema)
     else_ = schema.pop("else", schema)
-    if if_ is not None and (then is not schema or else_ is not schema):
-        if then not in (if_, TRUTHY) or else_ != TRUTHY:
-            alternatives = [
-                {"allOf": [if_, then, schema]},
-                {"allOf": [{"not": if_}, else_, schema]},
-            ]
-            schema = canonicalish({"anyOf": alternatives})
+    if (
+        if_ is not None
+        and (then is not schema or else_ is not schema)
+        and (then not in (if_, TRUTHY) or else_ != TRUTHY)
+    ):
+        alternatives = [
+            {"allOf": [if_, then, schema]},
+            {"allOf": [{"not": if_}, else_, schema]},
+        ]
+        schema = canonicalish({"anyOf": alternatives})
     assert isinstance(schema, dict)
     # Recurse into the value of each keyword with a schema (or list of them) as a value
     for key in SCHEMA_KEYS:
@@ -385,14 +391,20 @@ def canonicalish(schema: JSONType) -> Dict[str, Any]:
     ):
         max_props = schema.get("maxProperties", math.inf)
         assert isinstance(max_props, (int, float))
+        for k, v in list(schema["properties"].items()):
+            if v == FALSEY:
+                schema["properties"].pop(k)
         schema["maxProperties"] = min(max_props, len(schema["properties"]))
+    if schema.get("maxProperties", math.inf) == 0:
+        for k in ("properties", "patternProperties", "additionalProperties"):
+            schema.pop(k, None)
     if "object" in type_ and schema.get("minProperties", 0) > schema.get(
         "maxProperties", math.inf
     ):
         type_.remove("object")
     # Discard dependencies values that don't restrict anything
     for k, v in schema.get("dependencies", {}).copy().items():
-        if v == [] or v == TRUTHY:
+        if v in ([], TRUTHY):
             schema["dependencies"].pop(k)
     # Remove no-op keywords
     for kw, identity in {
@@ -435,15 +447,13 @@ def canonicalish(schema: JSONType) -> Dict[str, Any]:
         max_ = schema.get("maxProperties", float("inf"))
         assert isinstance(max_, (int, float))
         properties = schema.get("properties", {})
-        if len(schema["required"]) > max_:
+        propnames_validator = make_validator(schema.get("propertyNames", {})).is_valid
+        if (
+            len(schema["required"]) > max_
+            or any(properties.get(name, {}) == FALSEY for name in schema["required"])
+            or not all(propnames_validator(name) for name in schema["required"])
+        ):
             type_.remove("object")
-        elif any(properties.get(name, {}) == FALSEY for name in schema["required"]):
-            type_.remove("object")
-        else:
-            propnames = schema.get("propertyNames", {})
-            validator = make_validator(propnames)
-            if not all(validator.is_valid(name) for name in schema["required"]):
-                type_.remove("object")
 
     for t, kw in TYPE_SPECIFIC_KEYS:
         numeric = {"number", "integer"}
@@ -716,7 +726,7 @@ def merged(schemas: List[Any]) -> Optional[Schema]:
         if "contains" in out and "contains" in s and out["contains"] != s["contains"]:
             # If one `contains` schema is a subset of the other, we can discard it.
             m = merged([out["contains"], s["contains"]])
-            if m == out["contains"] or m == s["contains"]:
+            if m in (out["contains"], s["contains"]):
                 out["contains"] = m
                 s.pop("contains")
         if "not" in out and "not" in s and out["not"] != s["not"]:
